@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005-2015 Fabrice Colin
+ *  Copyright 2005-2021 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -171,7 +171,7 @@ PinotSettings &PinotSettings::getInstance(void)
 	return m_instance;
 }
 
-bool PinotSettings::enableClientMode(bool enable)
+bool PinotSettings::setClientMode(bool enable)
 {
 	bool isEnabled = m_clientMode;
 
@@ -180,6 +180,11 @@ bool PinotSettings::enableClientMode(bool enable)
 	return isEnabled;
 }
 
+bool PinotSettings::getClientMode(void)
+{
+	return m_clientMode;
+}
+
 string PinotSettings::getHomeDirectory(void)
 {
 #ifdef HAVE_PWD_H
@@ -302,7 +307,6 @@ void PinotSettings::clear(void)
 	m_version.clear();
 	m_warnAboutVersion = false;
 	m_defaultBackend = "xapian";
-	m_googleAPIKey.clear();
 	m_xPos = 0;
 	m_yPos = 0;
 	m_width = 0;
@@ -474,11 +478,13 @@ bool PinotSettings::load(LoadWhat what)
 		QueryProperties queryProps(_("Latest First"), "dir:/");
 		queryProps.setSortOrder(QueryProperties::DATE_DESC);
 		addQuery(queryProps);
+		queryProps = QueryProperties(_("10kb And Smaller"), "0..10240b");
+		queryProps.setSortOrder(QueryProperties::SIZE_DESC);
+		addQuery(queryProps);
 		addQuery(QueryProperties(_("Home Stuff"), string("dir:") + getHomeDirectory()));
 		addQuery(QueryProperties(_("With Label New"), "label:New"));
 		addQuery(QueryProperties(_("Have CJKV"), "tokens:CJKV"));
 		addQuery(QueryProperties(_("In English"), "lang:en"));
-		addQuery(QueryProperties(_("10kb And Smaller"), "0..10240b"));
 		addQuery(QueryProperties("Pinot search", "pinot search"));
 	}
 
@@ -643,10 +649,6 @@ bool PinotSettings::loadConfiguration(const string &fileName, bool isGlobal)
 				{
 					m_defaultBackend = nodeContent;
 				}
-				else if (nodeName == "googleapikey")
-				{
-					m_googleAPIKey = nodeContent;
-				}
 				else if (nodeName == "ui")
 				{
 					loadUi(pElem);
@@ -926,6 +928,10 @@ bool PinotSettings::loadQueries(const Element *pElem)
 			{
 				queryProps.setSortOrder(QueryProperties::DATE_ASC);
 			}
+			else if (nodeContent == "SIZE_DESC")
+			{
+				queryProps.setSortOrder(QueryProperties::SIZE_DESC);
+			}
 			else
 			{
 				queryProps.setSortOrder(QueryProperties::RELEVANCE);
@@ -1561,6 +1567,10 @@ bool PinotSettings::save(SaveWhat what)
 				{
 					sortOrder = "DATE_ASC";
 				}
+				else if (queryIter->second.getSortOrder() == QueryProperties::SIZE_DESC)
+				{
+					sortOrder = "SIZE_DESC";
+				}
 				addChildElement(pElem, "name", queryIter->first);
 				addChildElement(pElem, "sortorder", sortOrder);
 				addChildElement(pElem, "text", queryIter->second.getFreeQuery());
@@ -1587,7 +1597,6 @@ bool PinotSettings::save(SaveWhat what)
 		if (what == SAVE_PREFS)
 		{
 			addChildElement(pRootElem, "backend", m_defaultBackend);
-			addChildElement(pRootElem, "googleapikey", m_googleAPIKey);
 			// Labels
 			for (set<string>::iterator labelIter = m_labels.begin(); labelIter != m_labels.end(); ++labelIter)
 			{
