@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005-2016 Fabrice Colin
+ *  Copyright 2005-2021 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -74,6 +74,9 @@ ustring QueueManager::index_document(const DocumentInfo &docInfo)
 {
 	string location(docInfo.getLocation());
 
+#ifdef DEBUG
+	clog << "ThreadsManager::index_document: called with " << location << endl;
+#endif
 	if (m_stopIndexing == true)
 	{
 #ifdef DEBUG
@@ -111,6 +114,7 @@ ustring QueueManager::index_document(const DocumentInfo &docInfo)
 		if (beingProcessed == true)
 		{
 			// FIXME: we may have to set labels on this document
+			// FIXME: fix this for RTL languages
 			ustring status(location);
 			status += " ";
 			status += _("is already being indexed");
@@ -121,6 +125,7 @@ ustring QueueManager::index_document(const DocumentInfo &docInfo)
 	// Is the document blacklisted ?
 	if (PinotSettings::getInstance().isBlackListed(location) == true)
 	{
+		// FIXME: fix this for RTL languages
 		ustring status(location);
 		status += " ";
 		status += _("is blacklisted");
@@ -130,8 +135,12 @@ ustring QueueManager::index_document(const DocumentInfo &docInfo)
 	if ((m_scanLocalFiles == true) &&
 		(urlObj.isLocal() == true))
 	{
+#ifdef DEBUG
+		clog << "ThreadsManager::index_document: scanning " <<
+			urlObj.getLocation() + "/" + urlObj.getFile() << endl;
+#endif
 		// This handles both directories and files
-		start_thread(new DirectoryScannerThread(urlObj.getLocation() + "/" + urlObj.getFile(),
+		start_thread(new DirectoryScannerThread(docInfo,
 			m_defaultIndexLocation, 0, true, true));
 	}
 	else
@@ -199,7 +208,7 @@ bool QueueManager::pop_queue(const string &urlWasIndexed)
 	bool emptyQueue = false;
 
 #ifdef DEBUG
-	clog << "QueueManager::pop_queue: called" << endl;
+	clog << "QueueManager::pop_queue: called with " << urlWasIndexed << endl;
 #endif
 	if (get_threads_count() >= m_maxIndexThreads)
 	{
@@ -248,6 +257,7 @@ bool QueueManager::pop_queue(const string &urlWasIndexed)
 				if (docInfo.getLocation() == previousLocation)
 				{
 					// Something dodgy is going on, we got the same item twice !
+					// FIXME: fix this for RTL languages
 					status = previousLocation;
 					status += " ";
 					status += _("is already being indexed");
@@ -796,16 +806,6 @@ IndexingThread::IndexingThread(const DocumentInfo &docInfo, const string &indexL
 {
 }
 
-IndexingThread::IndexingThread(const string &indexLocation) :
-	DownloadingThread(),
-	m_pIndex(NULL),
-	m_indexLocation(indexLocation),
-	m_allowAllMIMETypes(true),
-	m_update(false),
-	m_docId(0)
-{
-}
-
 IndexingThread::~IndexingThread()
 {
 	if (m_pIndex != NULL)
@@ -876,7 +876,14 @@ void IndexingThread::doWork(void)
 		reliableType = true;
 	}
 
-	if (FilterUtils::isSupportedType(m_docInfo.getType()) == false)
+	if (m_docInfo.getIsDirectory() == true)
+	{
+		doDownload = false;
+#ifdef DEBUG
+		clog << "IndexingThread::doWork: skipping download of directory " << m_docInfo.getLocation() << endl;
+#endif
+	}
+	else if (FilterUtils::isSupportedType(m_docInfo.getType()) == false)
 	{
 		// Skip unsupported types ?
 		if (m_allowAllMIMETypes == false)
@@ -1029,6 +1036,12 @@ void IndexingThread::doWork(void)
 			// Update an existing document or add to the index ?
 			if (m_update == true)
 			{
+				set<string> labels;
+
+				// Make sure labels are preserved
+				m_pIndex->getDocumentLabels(m_docId, labels);
+				m_pDoc->setLabels(labels);
+
 				// Update the document
 				if (wrapFilter.updateDocument(*m_pDoc, m_docId) == true)
 				{
@@ -1248,16 +1261,18 @@ void HistoryMonitorThread::fileModified(const string &location)
 #endif
 }
 
-DirectoryScannerThread::DirectoryScannerThread(const string &dirName,
+DirectoryScannerThread::DirectoryScannerThread(const DocumentInfo &docInfo,
 	const string &indexLocation, unsigned int maxLevel,
 	bool inlineIndexing, bool followSymLinks) :
-	IndexingThread(indexLocation),
-	m_dirName(dirName),
+	IndexingThread(docInfo, indexLocation),
 	m_currentLevel(0),
 	m_maxLevel(maxLevel),
 	m_inlineIndexing(inlineIndexing),
 	m_followSymLinks(followSymLinks)
 {
+	Url urlObj(docInfo.getLocation());
+
+	m_dirName = urlObj.getLocation() + "/" + urlObj.getFile();
 }
 
 DirectoryScannerThread::~DirectoryScannerThread()
@@ -1350,6 +1365,11 @@ bool DirectoryScannerThread::monitorEntry(const string &entryName)
 	return true;
 }
 
+void DirectoryScannerThread::unmonitorEntry(const string &entryName)
+{
+	// Nothing to do by default
+}
+
 void DirectoryScannerThread::foundFile(const DocumentInfo &docInfo)
 {
 	if ((docInfo.getLocation().empty() == true) ||
@@ -1373,25 +1393,22 @@ void DirectoryScannerThread::foundFile(const DocumentInfo &docInfo)
 	else
 	{
 		// Delegate indexing
+		// Report everything as file to avoid triggering another crawl
 		m_signalFileFound(docInfo, false);
 	}
 }
 
 bool DirectoryScannerThread::scanEntry(const string &entryName,
-	bool statLinks)
+	int &entryStatus, bool statLinks)
 {
 	string location("file://" + entryName);
 	DocumentInfo docInfo("", location, "", "");
 	time_t itemDate = time(NULL);
 	struct stat fileStat;
-	int entryStatus = 0;
 	bool scanSuccess = true, reportFile = false, itemExists = false;
 
 	if (entryName.empty() == true)
 	{
-#ifdef DEBUG
-		clog << "DirectoryScannerThread::scanEntry: no name" << endl;
-#endif
 		return false;
 	}
 
@@ -1510,7 +1527,7 @@ bool DirectoryScannerThread::scanEntry(const string &entryName,
 			recordSymlink(referreeLocation, itemDate);
 
 			// Do it again, this time by stat'ing what the link refers to
-			bool scannedReferree = scanEntry(entryName, false);
+			bool scannedReferree = scanEntry(entryName, entryStatus, false);
 
 			m_currentLinks.pop();
 			m_currentLinkReferrees.pop();
@@ -1562,9 +1579,6 @@ bool DirectoryScannerThread::scanEntry(const string &entryName,
 			DIR *pDir = opendir(entryName.c_str());
 			if (pDir != NULL)
 			{
-#ifdef DEBUG
-				clog << "DirectoryScannerThread::scanEntry: entering " << entryName << endl;
-#endif
 				// Monitor first so that we don't miss events
 				// If monitoring is not possible, record the first case
 				if ((monitorEntry(entryName) == false) &&
@@ -1572,6 +1586,9 @@ bool DirectoryScannerThread::scanEntry(const string &entryName,
 				{
 					entryStatus = MONITORING_FAILED;
 				}
+#ifdef DEBUG
+				clog << "DirectoryScannerThread::scanEntry: entering " << entryName << endl;
+#endif
 
 				// Iterate through this directory's entries
 				struct dirent *pDirEntry = readdir(pDir);
@@ -1585,6 +1602,7 @@ bool DirectoryScannerThread::scanEntry(const string &entryName,
 						(pEntryName[0] != '.'))
 					{
 						string subEntryName(entryName);
+						int subEntryStatus = 0;
 
 						if (entryName[entryName.length() - 1] != '/')
 						{
@@ -1593,7 +1611,7 @@ bool DirectoryScannerThread::scanEntry(const string &entryName,
 						subEntryName += pEntryName;
 
 						// Scan this entry
-						scanEntry(subEntryName);
+						scanEntry(subEntryName, subEntryStatus);
 					}
 
 					// Next entry
@@ -1638,7 +1656,7 @@ bool DirectoryScannerThread::scanEntry(const string &entryName,
 	{
 		// No, it wasn't
 #ifdef DEBUG
-		clog << "DirectoryScannerThread::scanEntry: not reporting " << location << endl;
+		clog << "DirectoryScannerThread::scanEntry: no change to " << location << endl;
 #endif
 		reportFile = false;
 	}
@@ -1688,6 +1706,7 @@ bool DirectoryScannerThread::scanEntry(const string &entryName,
 void DirectoryScannerThread::doWork(void)
 {
 	Timer scanTimer;
+	int entryStatus = 0;
 
 	if (m_dirName.empty() == true)
 	{
@@ -1695,9 +1714,16 @@ void DirectoryScannerThread::doWork(void)
 	}
 	scanTimer.start();
 
-	if (scanEntry(m_dirName) == false)
+	if (scanEntry(m_dirName, entryStatus) == false)
 	{
-		m_errorNum = OPENDIR_FAILED;
+		if (entryStatus == 0)
+		{
+			m_errorNum = OPENDIR_FAILED;
+		}
+		else
+		{
+			m_errorNum = entryStatus;
+		}
 		m_errorParam = m_dirName;
 	}
 	clog << "Scanned " << m_dirName << " in " << scanTimer.stop() << " ms" << endl;
