@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005-2019 Fabrice Colin
+ *  Copyright 2005-2021 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -278,13 +278,35 @@ bool SherlockResponseParser::parse(const Document *pResponseDoc, vector<Document
 		return false;
 	}
 
+	string listStart(m_resultListStart);
+	string listEnd(m_resultListEnd);
+	string resStart(m_resultItemStart);
+	string resEnd(m_resultItemEnd);
+
 	// Extract the results list
 #ifdef DEBUG
 	clog << "SherlockResponseParser::parse: getting results list ("
 		<< m_resultListStart << ", " << m_resultListEnd << ")" << endl;
 #endif
 	const char *pContent = pResponseDoc->getData(contentLen);
-	string resultList = StringManip::extractField(pContent, m_resultListStart, m_resultListEnd);
+	string resultList = StringManip::extractField(pContent,
+		listStart, listEnd);
+	if (resultList.empty() == true)
+	{
+		// The other quotes may be used
+		if (m_resultListStart.find("\"") != string::npos)
+		{
+			listStart = StringManip::replaceSubString(m_resultListStart, "\"", "'");
+		}
+		else if (m_resultListStart.find("'") != string::npos)
+		{
+			listStart = StringManip::replaceSubString(m_resultListStart, "'", "\"");
+		}
+
+		// Try again
+		resultList = StringManip::extractField(pContent,
+			listStart, listEnd);
+	}
 	if (resultList.empty() == true)
 	{
 		resultList = string(pContent, contentLen);
@@ -297,7 +319,23 @@ bool SherlockResponseParser::parse(const Document *pResponseDoc, vector<Document
 		<< m_resultItemStart << ", " << m_resultItemEnd << ")" << endl;
 #endif
 	string resultItem = StringManip::extractField(resultList,
-		m_resultItemStart, m_resultItemEnd, endPos);
+		resStart, resEnd, endPos);
+	if (resultItem.empty() == true)
+	{
+		// The other quotes may be used
+		if (m_resultItemStart.find("\"") != string::npos)
+		{
+			resStart = StringManip::replaceSubString(m_resultItemStart, "\"", "'");
+		}
+		else if (m_resultItemStart.find("'") != string::npos)
+		{
+			resStart = StringManip::replaceSubString(m_resultItemStart, "'", "\"");
+		}
+
+		// Try again
+		resultItem = StringManip::extractField(resultList,
+			resStart, resEnd, endPos);
+	}
 	while (resultItem.empty() == false)
 	{
 		string contentType, url, name, extract;
@@ -441,7 +479,7 @@ bool SherlockResponseParser::parse(const Document *pResponseDoc, vector<Document
 		// Next
 		endPos += m_resultItemEnd.length();
 		resultItem = StringManip::extractField(resultList,
-			m_resultItemStart, m_resultItemEnd, endPos);
+			resStart, resEnd, endPos);
 	}
 
 	return foundResult;
