@@ -45,8 +45,8 @@ extern "C"
 #include "LanguageDetector.h"
 #include "config.h"
 
-using std::cout;
-using std::cerr;
+using std::clog;
+using std::clog;
 using std::endl;
 using std::string;
 using std::vector;
@@ -66,18 +66,23 @@ LanguageDetector::LanguageDetector() :
 	// What configuration file should we use ?
 	confFile += "/pinot/";
 #ifdef DEBUG
-	cout << "LanguageDetector::guessLanguage: detected " << textCatVersion << endl;
+	clog << "LanguageDetector::guessLanguage: detected " << textCatVersion << endl;
 #endif
-	if (strncasecmp(textCatVersion, "3.1.0", 9) == 0)
-	{
-		// Version 3
-		confFile += "textcat31_conf.txt";
-	}
-	else if (strncasecmp(textCatVersion, "TextCat 3", 9) == 0)
+	if (strncasecmp(textCatVersion, "TextCat 3", 9) == 0)
 	{
 		// Version 3
 		confFile += "textcat3_conf.txt";
 	}
+	else if (strncasecmp(textCatVersion, "3.1", 3) == 0)
+	{
+		// Version 3.1
+		confFile += "textcat31_conf.txt";
+	}
+	else if (strncasecmp(textCatVersion, "3.", 2) == 0)
+	{
+		// Version 3.2 and above
+		confFile += "textcat32_conf.txt";
+	}
 	else
 	{
 		confFile += "textcat_conf.txt";
@@ -156,7 +161,7 @@ void LanguageDetector::guessLanguage(const char *pData, unsigned int dataLength,
 				language.resize(dashPos);
 			}
 #ifdef DEBUG
-			cout << "LanguageDetector::guessLanguage: found language " << language << endl;
+			clog << "LanguageDetector::guessLanguage: found language " << language << endl;
 #endif
 			candidates.push_back(language);
 		}
@@ -198,7 +203,7 @@ void LanguageDetector::guessLanguage(const char *pData, unsigned int dataLength,
 					language.resize(dashPos);
 				}
 #ifdef DEBUG
-				cout << "LanguageDetector::guessLanguage: found language " << language << endl;
+				clog << "LanguageDetector::guessLanguage: found language " << language << endl;
 #endif
 				candidates.push_back(language);
 
@@ -212,7 +217,7 @@ void LanguageDetector::guessLanguage(const char *pData, unsigned int dataLength,
 	// Unlock
 	pthread_mutex_unlock(&m_mutex);
 #ifdef DEBUG
-	cout << "LanguageDetector::guessLanguage: language guessing took "
+	clog << "LanguageDetector::guessLanguage: language guessing took "
 		<< timer.stop() << " ms" << endl;
 #endif
 #else
