@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005-2019 Fabrice Colin
+ *  Copyright 2005-2021 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include <time.h>
 #include <string>
 #include <cstring>
+#include <map>
 #include <vector>
 #include <iostream>
 #include <fstream>
@@ -36,10 +37,10 @@
 #include "CJKVTokenizer.h"
 #include "FieldMapperInterface.h"
 #include "XapianDatabaseFactory.h"
-#include "AbstractGenerator.h"
 #include "XapianEngine.h"
 
 using std::string;
+using std::map;
 using std::multimap;
 using std::vector;
 using std::clog;
@@ -91,31 +92,19 @@ static void checkFilter(const string &freeQuery, string::size_type filterValueSt
 	}
 }
 
-#if XAPIAN_NUM_VERSION >= 1003006
 class TimeValueRangeProcessor : public Xapian::RangeProcessor
-#else
-class TimeValueRangeProcessor : public Xapian::ValueRangeProcessor
-#endif
 {
 	public:
 		TimeValueRangeProcessor(Xapian::valueno valueNumber) :
-#if XAPIAN_NUM_VERSION >= 1003006
 			Xapian::RangeProcessor(),
-#else
-			Xapian::ValueRangeProcessor(),
-#endif
 			m_valueNumber(valueNumber)
 		{
 		}
-		~TimeValueRangeProcessor()
+		virtual ~TimeValueRangeProcessor()
 		{
 		}
 
-#if XAPIAN_NUM_VERSION >= 1003006
 		virtual Xapian::Query operator()(const std::string &begin, const std::string &end)
-#else
-		virtual Xapian::valueno operator()(string &begin, string &end)
-#endif
 		{
 			if ((begin.size() == 6) &&
 				(end.size() == 6))
@@ -125,13 +114,9 @@ class TimeValueRangeProcessor : public Xapian::ValueRangeProcessor
 				clog << "TimeValueRangeProcessor::operator: accepting " << begin << ".." << end << endl;
 #endif
 
-#if XAPIAN_NUM_VERSION >= 1003006
 				return Xapian::Query(Xapian::Query::OP_VALUE_RANGE,
 					m_valueNumber,
 					begin,end);
-#else
-				return m_valueNumber;
-#endif
 			}
 			if ((begin.size() == 8) && (end.size() == 8) &&
 				(begin[2] == begin[5]) && (end[2] == end[5]) && (begin[2] == end[2]) &&
@@ -148,26 +133,15 @@ class TimeValueRangeProcessor : public Xapian::ValueRangeProcessor
 				clog << "TimeValueRangeProcessor::operator: accepting " << lower << ".." << upper << endl;
 #endif
 
-#if XAPIAN_NUM_VERSION >= 1003006
 				return Xapian::Query(Xapian::Query::OP_VALUE_RANGE,
 					m_valueNumber,
 					lower, upper);
-#else
-				begin = lower;
-				end = upper;
-
-				return m_valueNumber;
-#endif
 			}
 #ifdef DEBUG
 			clog << "TimeValueRangeProcessor::operator: rejecting " << begin << ".." << end << endl;
 #endif
 
-#if XAPIAN_NUM_VERSION >= 1003006
 			return Xapian::Query(Xapian::Query::OP_INVALID);
-#else
-			return Xapian::BAD_VALUENO;
-#endif
 		}
 
 	protected:
@@ -215,7 +189,7 @@ class TermDecider : public Xapian::ExpandDecider
 			clog << "TermDecider: avoiding " << m_pTermsToAvoid->size() << " terms" << endl;
 #endif
 		}
-		~TermDecider()
+		virtual ~TermDecider()
 		{
 			if (m_pTermsToAvoid != NULL)
 			{
@@ -396,9 +370,8 @@ class QueryModifier : public Dijon::CJKVTokenizer::TokensHandler
 		typedef enum { NONE = 0, BRACKETS } CJKVWrap;
 
 		QueryModifier(const string &query,
-			bool diacriticSensitive, unsigned int nGramSize) :
+			unsigned int nGramSize) :
 			m_query(query),
-			m_diacriticSensitive(diacriticSensitive),
 			m_pos(0),
 			m_wrap(BRACKETS),
 			m_wrapped(false),
@@ -409,7 +382,6 @@ class QueryModifier : public Dijon::CJKVTokenizer::TokensHandler
 			m_hasNonCJKV(false)
 		{
 		}
-
 		virtual ~QueryModifier()
 		{
 		}
@@ -469,17 +441,14 @@ class QueryModifier : public Dijon::CJKVTokenizer::TokensHandler
 					m_hasNonCJKV = true;
 				}
 
-				if (m_diacriticSensitive == false)
+				// Strip accents and other diacritics from terms
+				string unaccentedTok(Dijon::CJKVTokenizer::strip_marks(tok));
+				if (tok != unaccentedTok)
 				{
-					// Strip accents and other diacritics from terms
-					string unaccentedTok(Dijon::CJKVTokenizer::strip_marks(tok));
-					if (tok != unaccentedTok)
-					{
 #ifdef DEBUG
-						clog << "QueryModifier::handle_token: " << tok << " stripped to " << unaccentedTok << endl;
+					clog << "QueryModifier::handle_token: " << tok << " stripped to " << unaccentedTok << endl;
 #endif
-						m_query.replace(tokPos, tok.length(), unaccentedTok);
-					}
+					m_query.replace(tokPos, tok.length(), unaccentedTok);
 				}
 
 				// Return right away
@@ -563,7 +532,6 @@ class QueryModifier : public Dijon::CJKVTokenizer::TokensHandler
 
 	protected:
 		string m_query;
-		bool m_diacriticSensitive;
 		string m_modifiedQuery;
 		string::size_type m_pos;
 		CJKVWrap m_wrap;
@@ -640,45 +608,23 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 	CJKVTokenizer tokenizer;
 	string freeQuery(queryProps.getFreeQuery());
 	unsigned int tokensCount = 1;
-	bool diacriticSensitive = queryProps.getDiacriticSensitive();
 
-	// Modifying the query is necessary if it's CJKV or diacritics are off
-	if ((tokenizer.has_cjkv(freeQuery) == true) ||
-		(diacriticSensitive == false))
-	{
-		QueryModifier handler(freeQuery,
-			diacriticSensitive,
-			tokenizer.get_ngram_size());
+	// Modifying the query is necessary as diacritics sensitivity is off
+	QueryModifier handler(freeQuery,
+		tokenizer.get_ngram_size());
 
-		tokenizer.tokenize(freeQuery, handler, true);
+	tokenizer.tokenize(freeQuery, handler, true);
 
-		tokensCount = handler.get_tokens_count();
+	tokensCount = handler.get_tokens_count();
 
-		// We can disable stemming and spelling correction for pure CJKV queries
-		string cjkvQuery(handler.get_modified_query(minimal));
+	// We can disable stemming and spelling correction for pure CJKV queries
+	string cjkvQuery(handler.get_modified_query(minimal));
 #ifdef DEBUG
-		clog << "XapianEngine::parseQuery: CJKV query is " << cjkvQuery << endl;
+	clog << "XapianEngine::parseQuery: CJKV query is " << cjkvQuery << endl;
 #endif
 
-		// Do as if the user had given this as input
-		freeQuery = cjkvQuery;
-	}
-	else
-	{
-		string::size_type spacePos = freeQuery.find(' ');
-		while (spacePos != string::npos)
-		{
-			++tokensCount;
-
-			if (spacePos + 1 >= freeQuery.length())
-			{
-				break;
-			}
-
-			// Next
-			spacePos = freeQuery.find(' ', spacePos + 1);
-		}
-	}
+	// Do as if the user had given this as input
+	freeQuery = cjkvQuery;
 #ifdef DEBUG
 	clog << "XapianEngine::parseQuery: " << tokensCount << " tokens" << endl;
 #endif
@@ -723,15 +669,13 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 	{
 		parser.set_default_op(Xapian::Query::OP_OR);
 	}
-#if XAPIAN_NUM_VERSION >= 1000004
 	// Search across text body and title
 	parser.add_prefix("", "");
 	parser.add_prefix("", "S");
-#endif
 	// X prefixes should always include a colon
 	parser.add_boolean_prefix("site", "H");
 	parser.add_boolean_prefix("file", "P");
-	parser.add_boolean_prefix("ext", "XEXT:");
+	parser.add_boolean_prefix("ext", "E");
 	parser.add_prefix("title", "S");
 	parser.add_boolean_prefix("url", "U");
 	parser.add_boolean_prefix("dir", "XDIR:");
@@ -756,42 +700,16 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 	}
 
 	// Date range
-#if XAPIAN_NUM_VERSION >= 1003006
 	Xapian::DateRangeProcessor dateProcessor(0);
 	parser.add_rangeprocessor(&dateProcessor);
-#else
-	Xapian::DateValueRangeProcessor dateProcessor(0);
-	parser.add_valuerangeprocessor(&dateProcessor);
-#endif
 
 	// Size with a "b" suffix, ie 1024..10240b
-#if XAPIAN_NUM_VERSION >= 1003006
-	Xapian::NumberRangeProcessor sizeProcessor(2, "b", false);
+	Xapian::NumberRangeProcessor sizeProcessor(2, "b", Xapian::RP_SUFFIX);
 	parser.add_rangeprocessor(&sizeProcessor);
-#elif XAPIAN_NUM_VERSION >= 1001000
-	Xapian::NumberValueRangeProcessor sizeProcessor(2, "b", false);
-	parser.add_valuerangeprocessor(&sizeProcessor);
-#elif XAPIAN_NUM_VERSION >= 1000002
-	// Xapian 1.02 is the bare minimum
-	Xapian::v102::NumberValueRangeProcessor sizeProcessor(2, "b", false);
-	parser.add_valuerangeprocessor(&sizeProcessor);
-#endif
 
 	// Time range
 	TimeValueRangeProcessor timeProcessor(3);
-#if XAPIAN_NUM_VERSION >= 1003006
 	parser.add_rangeprocessor(&timeProcessor);
-#else
-	parser.add_valuerangeprocessor(&timeProcessor);
-#endif
-
-	// What type of query is this ?
-	QueryProperties::QueryType type = queryProps.getType();
-	if (type != QueryProperties::XAPIAN_QP)
-	{
-		// This isn't supported
-		return Xapian::Query();
-	}
 
 	// Do some pre-processing : look for filters with quoted values
 	string::size_type escapedFilterEnd = 0;
@@ -856,9 +774,7 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 	if (minimal == false)
 	{
 		flags |= Xapian::QueryParser::FLAG_WILDCARD;
-#if ENABLE_XAPIAN_SPELLING_CORRECTION>0
 		flags |= Xapian::QueryParser::FLAG_SPELLING_CORRECTION;
-#endif
 	}
 	Xapian::Query parsedQuery = parser.parse_query(freeQuery, flags);
 #ifdef DEBUG
@@ -880,7 +796,6 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 
 	if (minimal == false)
 	{
-#if ENABLE_XAPIAN_SPELLING_CORRECTION>0
 		// Any correction ?
 		correctedFreeQuery = parser.get_corrected_query_string();
 #ifdef DEBUG
@@ -888,7 +803,6 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 		{
 			clog << "XapianEngine::parseQuery: corrected spelling to: " << correctedFreeQuery << endl;
 		}
-#endif
 #endif
 	}
 
@@ -913,20 +827,13 @@ bool XapianEngine::queryDatabase(Xapian::Database *pIndex, Xapian::Query &query,
 	timer.start();
 	try
 	{
-		AbstractGenerator abstractGen(pIndex, 50);
-		vector<string> seedTerms;
-
 		// Give the query object to the enquire session
 		enquire.set_query(query);
 		// How should results be sorted ?
 		if (queryProps.getSortOrder() == QueryProperties::RELEVANCE)
 		{
 			// By relevance, then date
-#if XAPIAN_NUM_VERSION >= 1001000
 			enquire.set_sort_by_relevance_then_value(4, true);
-#else
-			enquire.set_sort_by_relevance_then_value(4);
-#endif
 #ifdef DEBUG
 			clog << "XapianEngine::queryDatabase: sorting by relevance first" << endl;
 #endif
@@ -935,11 +842,7 @@ bool XapianEngine::queryDatabase(Xapian::Database *pIndex, Xapian::Query &query,
 		{
 			// By date, and then by relevance
 			enquire.set_docid_order(Xapian::Enquire::DONT_CARE);
-#if XAPIAN_NUM_VERSION >= 1001000
 			enquire.set_sort_by_value_then_relevance(4, true);
-#else
-			enquire.set_sort_by_value_then_relevance(4);
-#endif
 #ifdef DEBUG
 			clog << "XapianEngine::queryDatabase: sorting by date and time desc" << endl;
 #endif
@@ -948,15 +851,20 @@ bool XapianEngine::queryDatabase(Xapian::Database *pIndex, Xapian::Query &query,
 		{
 			// By date, and then by relevance
 			enquire.set_docid_order(Xapian::Enquire::DONT_CARE);
-#if XAPIAN_NUM_VERSION >= 1001000
 			enquire.set_sort_by_value_then_relevance(5, true);
-#else
-			enquire.set_sort_by_value_then_relevance(5);
-#endif
 #ifdef DEBUG
 			clog << "XapianEngine::queryDatabase: sorting by date and time asc" << endl;
 #endif
 		}
+		else if (queryProps.getSortOrder() == QueryProperties::SIZE_DESC)
+		{
+			// By date, and then by relevance
+			enquire.set_docid_order(Xapian::Enquire::DONT_CARE);
+			enquire.set_sort_by_value_then_relevance(2, true);
+#ifdef DEBUG
+			clog << "XapianEngine::queryDatabase: sorting by size asc" << endl;
+#endif
+		}
 
 		// Collapse results ?
 		if (g_pMapper != NULL)
@@ -987,49 +895,7 @@ bool XapianEngine::queryDatabase(Xapian::Database *pIndex, Xapian::Query &query,
 			{
 				Xapian::docid docId = *mIter;
 				Xapian::Document doc(mIter.get_document());
-
-				// What terms did this document match ?
-				seedTerms.clear();
-				for (Xapian::TermIterator termIter = enquire.get_matching_terms_begin(docId);
-					termIter != enquire.get_matching_terms_end(docId); ++termIter)
-				{
-					char firstChar = (*termIter)[0];
-
-					if (isupper(((int)firstChar)) == 0)
-					{
-						seedTerms.push_back(*termIter);
-#ifdef DEBUG
-						clog << "XapianEngine::queryDatabase: matched term " << *termIter << endl;
-#endif
-					}
-					else if (firstChar == 'Z')
-					{
-						string stemmed((*termIter).substr(1));
-						string::size_type stemmedLen = stemmed.length();
-
-						// Which of this document's terms stem to this ?
-						Xapian::TermIterator docTermIter = pIndex->termlist_begin(docId);
-						if (docTermIter != pIndex->termlist_end(docId))
-						{
-							for (docTermIter.skip_to(stemmed);
-								docTermIter != pIndex->termlist_end(docId); ++docTermIter)
-							{
-								// Is this a potential unstem ?
-								if (strncasecmp((*docTermIter).c_str(), stemmed.c_str(), stemmedLen) != 0)
-								{
-									// No, no point looking at the next terms
-									break;
-								}
-#ifdef DEBUG
-								clog << "XapianEngine::queryDatabase: matched unstem " << *docTermIter << endl;
-#endif
-
-								// FIXME: check this term stems to stemmed !
-								seedTerms.push_back(*docTermIter); 
-							}
-						}
-					}
-				}
+				bool hasCJKV = false;
 
 				if (docId <= 0)
 				{
@@ -1040,7 +906,21 @@ bool XapianEngine::queryDatabase(Xapian::Database *pIndex, Xapian::Query &query,
 				}
 
 				DocumentInfo thisResult;
-				thisResult.setExtract(abstractGen.generateAbstract(docId, seedTerms));
+				string docText(getDocumentText(pIndex, docId, hasCJKV));
+				unsigned int flags = Xapian::MSet::SNIPPET_BACKGROUND_MODEL|Xapian::MSet::SNIPPET_EXHAUSTIVE;
+
+				if (hasCJKV == true)
+				{
+					flags |= Xapian::MSet::SNIPPET_CJK_NGRAM;
+				}
+				if (stemLanguage.empty() == true)
+				{
+					thisResult.setExtract(matches.snippet(docText, 300, Xapian::Stem(), flags));
+				}
+				else
+				{
+					thisResult.setExtract(matches.snippet(docText, 300, m_stemmer, flags));
+				}
 				thisResult.setScore((float)mIter.get_percent());
 
 #ifdef DEBUG
@@ -1145,6 +1025,70 @@ void XapianEngine::freeAll(void)
 	FileStopper::free_stopper();
 }
 
+string XapianEngine::getDocumentText(Xapian::Database *pIndex,
+	Xapian::docid docId, bool &hasCJKV)
+{
+	map<Xapian::termpos, string> wordsBuffer;
+	CJKVTokenizer tokenizer;
+
+	try
+	{
+		// Go through the position list of each term
+		for (Xapian::TermIterator termIter = pIndex->termlist_begin(docId);
+			termIter != pIndex->termlist_end(docId); ++termIter)
+		{
+			string termName(*termIter);
+
+			// Skip prefixed terms
+			if (isupper((int)termName[0]) != 0)
+			{
+				if (termName == "XTOK:CJKV")
+				{
+					hasCJKV = true;
+				}
+				continue;
+			}
+			// Skip multi-character CJKV terms
+			if ((tokenizer.has_cjkv(termName) == true) &&
+				(termName.length() > 4))
+			{
+				continue;
+			}
+
+			for (Xapian::PositionIterator positionIter = pIndex->positionlist_begin(docId, termName);
+				positionIter != pIndex->positionlist_end(docId, termName); ++positionIter)
+			{
+				Xapian::termpos termPos = *positionIter;
+
+				// If several terms exist at this position, prefer the shortest one
+				map<Xapian::termpos, string>::const_iterator wordIter = wordsBuffer.find(termPos);
+				if ((wordIter == wordsBuffer.end()) ||
+					(wordIter->second.length() > termName.length()))
+				{
+					wordsBuffer[termPos] = termName;
+				}
+			}
+		}
+	}
+	catch (const Xapian::Error &error)
+	{
+#ifdef DEBUG
+		clog << "XapianEngine::getDocumentText: " << error.get_msg() << endl;
+#endif
+	}
+
+	string docText;
+
+	for (map<Xapian::termpos, string>::const_iterator wordIter = wordsBuffer.begin();
+		wordIter != wordsBuffer.end(); ++wordIter)
+	{
+		docText += " ";
+		docText += wordIter->second;
+	}
+
+	return docText;
+}
+
 //
 // Implementation of SearchEngineInterface
 //
@@ -1206,7 +1150,8 @@ bool XapianEngine::runQuery(QueryProperties& queryProps,
 		return false;
 	}
 
-	if (stemLanguage.empty() == false)
+	if ((stemLanguage.empty() == false) &&
+		(stemLanguage != "unknown"))
 	{
 #ifdef DEBUG
 		clog << "XapianEngine::runQuery: " << stemLanguage << " stemming" << endl;
