@@ -1,5 +1,5 @@
 /*
- *  Copyright 2008-2016 Fabrice Colin
+ *  Copyright 2008-2021 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -60,10 +60,6 @@ class SQLResults
 
 		virtual bool rewind(void);
 
-		virtual int getIntCount(void);
-
-		virtual off_t getOffsetCount(void);
-
 	protected:
 		unsigned long m_nRows;
 		unsigned int m_nColumns;
