@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005-2012 Fabrice Colin
+ *  Copyright 2005-2014 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,14 +20,15 @@
 #define _SQLITE_BASE_H
 
 #include <string>
-#include <vector>
 #include <map>
+#include <vector>
+#include <utility>
 
 #include <sqlite3.h>
 
 #include "SQLDB.h"
 
-/// A row in a SQLite table.
+/// A row of results.
 class SQLiteRow : public SQLRow
 {
 	public:
@@ -41,6 +42,10 @@ class SQLiteRow : public SQLRow
 		std::vector<std::string> m_columns;
 		sqlite3_stmt *m_pStatement;
 
+	private:
+		SQLiteRow(const SQLiteRow &other);
+		SQLiteRow &operator=(const SQLiteRow &other);
+
 };
 
 /// Results extracted from a SQLite database.
@@ -86,8 +91,13 @@ class SQLiteBase : public SQLDB
 
 		static bool check(const std::string &databaseName);
 
+		bool backup(const std::string &destDatabaseName,
+			int pagesCount = 5, bool retryOnLock = true);
+
 		virtual bool isOpen(void) const;
 
+		virtual bool reopen(const std::string &databaseName);
+
 		virtual bool alterTable(const std::string &tableName,
 			const std::string &columns,
 			const std::string &newDefinition);
@@ -108,6 +118,9 @@ class SQLiteBase : public SQLDB
 		virtual SQLResults *executePreparedStatement(const std::string &statementId,
 			const std::vector<std::string> &values);
 
+		virtual SQLResults *executePreparedStatement(const std::string &statementId,
+			const std::vector<std::pair<std::string, SQLRow::SQLType> > &values);
+
 	protected:
 		bool m_onDemand;
 		bool m_inTransaction;
