@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2012 Fabrice Colin
+ *  Copyright 2007-2016 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -46,12 +46,12 @@ using std::pair;
 using namespace Dijon;
 
 #ifdef _DYNAMIC_DIJON_FILTERS
-DIJON_FILTER_EXPORT bool get_filter_types(set<string> &mime_types)
+DIJON_FILTER_EXPORT bool get_filter_types(MIMETypes &mime_types)
 {
-	mime_types.clear();
-	mime_types.insert("application/mbox");
-	mime_types.insert("text/x-mail");
-	mime_types.insert("text/x-news");
+	mime_types.m_mimeTypes.clear();
+	mime_types.m_mimeTypes.insert("application/mbox");
+	mime_types.m_mimeTypes.insert("text/x-mail");
+	mime_types.m_mimeTypes.insert("text/x-news");
 
 	return true;
 }
@@ -69,18 +69,18 @@ DIJON_FILTER_EXPORT bool check_filter_data_input(int data_input)
 	return false;
 }
 
-DIJON_FILTER_EXPORT Filter *get_filter(const string &mime_type)
+DIJON_FILTER_EXPORT Filter *get_filter(void)
 {
-	return new GMimeMboxFilter(mime_type);
+	return new GMimeMboxFilter();
 }
 
 DIJON_FILTER_INITIALIZE void initialize_gmime(void)
 {
 	// Initialize gmime
-#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
-	g_mime_init(GMIME_ENABLE_RFC2047_WORKAROUNDS);
+#if GMIME_MAJOR_VERSION >= 3
+	g_mime_init();
 #else
-	g_mime_init(GMIME_INIT_FLAG_UTF8);
+	g_mime_init(GMIME_ENABLE_RFC2047_WORKAROUNDS);
 #endif
 }
 
@@ -146,8 +146,8 @@ GMimeMboxFilter::GMimeMboxPart::~GMimeMboxPart()
 {
 }
 
-GMimeMboxFilter::GMimeMboxFilter(const string &mime_type) :
-	Filter(mime_type),
+GMimeMboxFilter::GMimeMboxFilter() :
+	Filter(),
 	m_returnHeaders(false),
 	m_maxSize(0),
 	m_pData(NULL),
@@ -211,7 +211,7 @@ bool GMimeMboxFilter::set_property(Properties prop_name, const string &prop_valu
 	return false;
 }
 
-bool GMimeMboxFilter::set_document_data(const char *data_ptr, unsigned int data_length)
+bool GMimeMboxFilter::set_document_data(const char *data_ptr, off_t data_length)
 {
 	// Close/free whatever was opened/allocated on a previous call to set_document()
 	finalize(true);
@@ -301,7 +301,7 @@ bool GMimeMboxFilter::skip_to_document(const string &ipath)
 	}
 
 	// ipath's format is "o=offset&l=part_levels"
-	if (sscanf(ipath.c_str(), "o="GMIME_OFFSET_MODIFIER"&l=[", &m_messageStart) != 1)
+	if (sscanf(ipath.c_str(), "o=" GMIME_OFFSET_MODIFIER "&l=[", &m_messageStart) != 1)
 	{
 		return false;
 	}
@@ -478,7 +478,11 @@ bool GMimeMboxFilter::initialize(void)
 		g_mime_parser_init_with_stream(m_pParser, m_pGMimeMboxStream);
 		g_mime_parser_set_respect_content_length(m_pParser, TRUE);
 		// Scan for mbox From-lines
+#if GMIME_MAJOR_VERSION >= 3
+		g_mime_parser_set_format(m_pParser, GMIME_FORMAT_MBOX);
+#else
 		g_mime_parser_set_scan_from(m_pParser, TRUE);
+#endif
 
 		return true;
 	}
@@ -493,14 +497,10 @@ void GMimeMboxFilter::finalize(bool fullReset)
 {
 	if (m_pMimeMessage != NULL)
 	{
-#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
 		if (G_IS_OBJECT(m_pMimeMessage))
 		{
 			g_object_unref(m_pMimeMessage);
 		}
-#else
-		g_mime_object_unref(GMIME_OBJECT(m_pMimeMessage));
-#endif
 		m_pMimeMessage = NULL;
 	}
 	if (m_pParser != NULL)
@@ -569,7 +569,7 @@ bool GMimeMboxFilter::readStream(GMimeStream *pStream, dstring &fileBuffer)
 		}
 		else if (bytesRead == -1)
 		{
-			// An error occured
+			// An error occurred
 			if (errno != EINTR)
 			{
 				gotOutput = false;
@@ -603,33 +603,14 @@ bool GMimeMboxFilter::nextPart(const string &subject)
 			if (extractPart(pMimePart, mboxPart) == true)
 			{
 				extractMetaData(mboxPart);
-
-#ifndef GMIME_ENABLE_RFC2047_WORKAROUNDS
-				if (pMimePart != NULL)
-				{
-					g_mime_object_unref(pMimePart);
-				}
-#endif
-
 				return true;
 			}
-
-#ifndef GMIME_ENABLE_RFC2047_WORKAROUNDS
-			if (pMimePart != NULL)
-			{
-				g_mime_object_unref(pMimePart);
-			}
-#endif
 		}
 
-#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
 		if (G_IS_OBJECT(m_pMimeMessage))
 		{
 			g_object_unref(m_pMimeMessage);
 		}
-#else
-		g_mime_object_unref(GMIME_OBJECT(m_pMimeMessage));
-#endif
 		m_pMimeMessage = NULL;
 	}
 
@@ -654,9 +635,6 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 #endif
 		GMimeMessage *partMessage = g_mime_message_part_get_message(GMIME_MESSAGE_PART(part));
 		part = g_mime_message_get_mime_part(partMessage);
-#ifndef GMIME_ENABLE_RFC2047_WORKAROUNDS
-		g_mime_object_unref(GMIME_OBJECT(partMessage));
-#endif
 	}
 
 	// Is this a multipart ?
@@ -665,11 +643,7 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 		int partsCount = 0, partNum = 0;
 		bool gotPart = false;
 
-#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
 		m_partsCount = partsCount = g_mime_multipart_get_count(GMIME_MULTIPART(part));
-#else
-		m_partsCount = partsCount = g_mime_multipart_get_number(GMIME_MULTIPART(part));
-#endif
 		++m_currentLevel;
 #ifdef DEBUG
 		clog << "GMimeMboxFilter::extractPart: message has " << m_partsCount << " parts at level " << m_currentLevel << endl;
@@ -709,9 +683,6 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 			}
 
 			gotPart = extractPart(multiMimePart, mboxPart);
-#ifndef GMIME_ENABLE_RFC2047_WORKAROUNDS
-			g_mime_object_unref(multiMimePart);
-#endif
 
 			if (gotPart == true)
 			{
@@ -770,13 +741,13 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 	GMimePart *mimePart = GMIME_PART(part);
 
 	// Check the content type
-#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
 	GMimeContentType *mimeType = g_mime_object_get_content_type(GMIME_OBJECT(mimePart));
-#else
-	const GMimeContentType *mimeType = g_mime_object_get_content_type(GMIME_OBJECT(mimePart));
-#endif
 	// Set this for caller
+#if GMIME_MAJOR_VERSION >= 3
+	char *partType = g_mime_content_type_get_mime_type(mimeType);
+#else
 	char *partType = g_mime_content_type_to_string(mimeType);
+#endif
 	if (partType != NULL)
 	{
 #ifdef DEBUG
@@ -842,19 +813,11 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 		return true;
 	}
 
-#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
 	GMimeContentEncoding encodingType = g_mime_part_get_content_encoding(mimePart);
-#else
-	GMimePartEncodingType encodingType = g_mime_part_get_encoding(mimePart);
-#endif
 #ifdef DEBUG
 	clog << "GMimeMboxFilter::extractPart: encoding is " << encodingType << endl;
 #endif
-#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
 	g_mime_part_set_content_encoding(mimePart, GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE);
-#else
-	g_mime_part_set_encoding(mimePart, GMIME_PART_ENCODING_QUOTEDPRINTABLE);
-#endif
 	const char *fileName = g_mime_part_get_filename(mimePart);
 	if (fileName != NULL)
 	{
@@ -893,7 +856,11 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 	}
 
 	// Write the part to the stream
+#if GMIME_MAJOR_VERSION >= 3
+	GMimeDataWrapper *dataWrapper = g_mime_part_get_content(mimePart);
+#else
 	GMimeDataWrapper *dataWrapper = g_mime_part_get_content_object(mimePart);
+#endif
 	if (dataWrapper != NULL)
 	{
 		ssize_t writeLen = g_mime_data_wrapper_write_to_stream(dataWrapper, memStream);
@@ -911,7 +878,11 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 		(mboxPart.m_contentType.length() >= 10) &&
 		(strncasecmp(mboxPart.m_contentType.c_str(), "text/plain", 10) == 0))
 	{
+#if GMIME_MAJOR_VERSION >= 3
+		char *pHeaders = g_mime_object_get_headers(GMIME_OBJECT(m_pMimeMessage), NULL);
+#else
 		char *pHeaders = g_mime_object_get_headers(GMIME_OBJECT(m_pMimeMessage));
+#endif
 
 		if (pHeaders != NULL)
 		{
@@ -932,6 +903,56 @@ bool GMimeMboxFilter::extractPart(GMimeObject *part, GMimeMboxPart &mboxPart)
 	return true;
 }
 
+bool GMimeMboxFilter::extractDate(const string &header)
+{
+	const char *pDate = g_mime_object_get_header(GMIME_OBJECT(m_pMimeMessage), header.c_str());
+
+	if (pDate == NULL)
+	{
+		return false;
+	}
+
+	string date(pDate);
+	struct tm timeTm;
+
+	timeTm.tm_sec = timeTm.tm_min = timeTm.tm_hour = timeTm.tm_mday = 0;
+	timeTm.tm_mon = timeTm.tm_year = timeTm.tm_wday = timeTm.tm_yday = timeTm.tm_isdst = 0;
+
+	if (date.find(',') != string::npos)
+	{
+		strptime(pDate, "%a, %d %b %Y %H:%M:%S %z", &timeTm);
+		if (timeTm.tm_year <= 0)
+		{
+			strptime(pDate, "%a, %d %b %y %H:%M:%S %z", &timeTm);
+		}
+	}
+	else
+	{
+		strptime(pDate, "%d %b %Y %H:%M:%S %z", &timeTm);
+		if (timeTm.tm_year <= 0)
+		{
+			strptime(pDate, "%d %b %y %H:%M:%S %z", &timeTm);
+		}
+	}
+
+	// Sanity check
+	if (timeTm.tm_year <= 0)
+	{
+#ifdef DEBUG
+		clog << "GMimeMboxFilter::extractDate: ignoring bogus year " << timeTm.tm_year << endl;
+#endif
+		return false;
+	}
+
+	m_messageDate = mktime(&timeTm);
+#ifdef DEBUG
+	clog << "GMimeMboxFilter::extractDate: message date is " << pDate
+		<< ": " << m_messageDate << endl;
+#endif
+
+	return true;
+}
+
 bool GMimeMboxFilter::extractMessage(const string &subject)
 {
 	string msgSubject(subject);
@@ -945,31 +966,31 @@ bool GMimeMboxFilter::extractMessage(const string &subject)
 			// No, it doesn't
 			if (m_pMimeMessage != NULL)
 			{
-#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
 				if (G_IS_OBJECT(m_pMimeMessage))
 				{
 					g_object_unref(m_pMimeMessage);
 				}
-#else
-				g_mime_object_unref(GMIME_OBJECT(m_pMimeMessage));
-#endif
 				m_pMimeMessage = NULL;
 			}
 
 			// Get the next message
+#if GMIME_MAJOR_VERSION >= 3
+			m_pMimeMessage = g_mime_parser_construct_message(m_pParser, NULL);
+#else
 			m_pMimeMessage = g_mime_parser_construct_message(m_pParser);
+#endif
 			if (m_pMimeMessage == NULL)
 			{
 				clog << "Couldn't construct new MIME message" << endl;
 				break;
 			}
 
-			m_messageStart = g_mime_parser_get_from_offset(m_pParser);
-#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
-			gint64 messageEnd = g_mime_parser_tell(m_pParser);
+#if GMIME_MAJOR_VERSION >= 3
+			m_messageStart = g_mime_parser_get_mbox_marker_offset(m_pParser);
 #else
-			off_t messageEnd = g_mime_parser_tell(m_pParser);
+			m_messageStart = g_mime_parser_get_from_offset(m_pParser);
 #endif
+			gint64 messageEnd = g_mime_parser_tell(m_pParser);
 #ifdef DEBUG
 			clog << "GMimeMboxFilter::extractMessage: message between offsets " << m_messageStart
 				<< " and " << messageEnd << endl;
@@ -1019,46 +1040,15 @@ bool GMimeMboxFilter::extractMessage(const string &subject)
 				}
 
 				// How old is this message ?
-				const char *pDate = g_mime_object_get_header(GMIME_OBJECT(m_pMimeMessage), "Date");
-				if (pDate == NULL)
+				if ((extractDate("Date") == false) &&
+					(extractDate("Delivery-Date") == false) &&
+					(extractDate("Resent-Date") == false))
 				{
-					pDate = g_mime_object_get_header(GMIME_OBJECT(m_pMimeMessage), "Resent-Date");
-				}
-				if (pDate != NULL)
-				{
-					m_messageDate = pDate;
-				}
-				else
-				{
-					time_t timeNow = time(NULL);
-					struct tm *pTimeTm = new struct tm;
-
-#ifdef HAVE_LOCALTIME_R
-					if (localtime_r(&timeNow, pTimeTm) != NULL)
-#else
-					pTimeTm = localtime(&timeNow);
-					if (pTimeTm != NULL)
-#endif
-					{
-						char timeStr[64];
-
-						// FIXME: don't use this extension ?
-#if defined(__GNU_LIBRARY__)
-						// %z is a GNU extension
-						if (strftime(timeStr, 64, "%a, %d %b %Y %H:%M:%S %z", pTimeTm) > 0)
-#else
-						if (strftime(timeStr, 64, "%a, %d %b %Y %H:%M:%S %Z", pTimeTm) > 0)
-#endif
-						{
-							m_messageDate = timeStr;
-						}
-					}
-
-					delete pTimeTm;
-				}
+					m_messageDate = time(NULL);
 #ifdef DEBUG
-				clog << "GMimeMboxFilter::extractMessage: message date is " << m_messageDate << endl;
+					clog << "GMimeMboxFilter::extractMessage: message date is today's " << m_messageDate << endl;
 #endif
+				}
 
 				// Extract the subject
 				const char *pSubject = g_mime_message_get_subject(m_pMimeMessage);
@@ -1104,7 +1094,7 @@ void GMimeMboxFilter::extractMetaData(GMimeMboxPart &mboxPart)
 	m_metaData["charset"] = m_partCharset;
 	snprintf(posStr, 128, "%lu", m_content.length());
 	m_metaData["size"] = posStr;
-	snprintf(posStr, 128, "o=%u&l=", m_messageStart);
+	snprintf(posStr, 128, "o=%ld&l=", m_messageStart);
 	ipath = posStr;
 	for (map<int, pair<int, int> >::const_iterator levelIter = m_levels.begin();
 		levelIter != m_levels.end(); ++levelIter)
