@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2012 Fabrice Colin
+ *  Copyright 2007-2016 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ namespace Dijon
     {
     public:
 	/// Builds an empty filter.
-	GMimeMboxFilter(const std::string &mime_type);
+	GMimeMboxFilter();
 	/// Destroys the filter.
 	virtual ~GMimeMboxFilter();
 
@@ -67,26 +67,26 @@ namespace Dijon
 	 * Filter object is destroyed, as some filters may not need to
 	 * do a deep copy of the data.
 	 * Call next_document() to position the filter onto the first document.
-	 * Returns false if this input is not supported or an error occured.
+	 * Returns false if this input is not supported or an error occurred.
 	 */
-	virtual bool set_document_data(const char *data_ptr, unsigned int data_length);
+	virtual bool set_document_data(const char *data_ptr, off_t data_length);
 
 	/** (Re)initializes the filter with the given data.
 	 * Call next_document() to position the filter onto the first document.
-	 * Returns false if this input is not supported or an error occured.
+	 * Returns false if this input is not supported or an error occurred.
 	 */
 	virtual bool set_document_string(const std::string &data_str);
 
 	/** (Re)initializes the filter with the given file.
 	 * Call next_document() to position the filter onto the first document.
-	 * Returns false if this input is not supported or an error occured.
+	 * Returns false if this input is not supported or an error occurred.
 	 */
 	virtual bool set_document_file(const std::string &file_path,
 		bool unlink_when_done = false);
 
 	/** (Re)initializes the filter with the given URI.
 	 * Call next_document() to position the filter onto the first document.
-	 * Returns false if this input is not supported or an error occured.
+	 * Returns false if this input is not supported or an error occurred.
 	 */
 	virtual bool set_document_uri(const std::string &uri);
 
@@ -95,7 +95,7 @@ namespace Dijon
 
 	/** Returns true if there are nested documents left to extract.
 	 * Returns false if the end of the parent document was reached
-	 * or an error occured.
+	 * or an error occurred.
 	 */
 	virtual bool has_documents(void) const;
 
@@ -112,7 +112,7 @@ namespace Dijon
 
 	// Accessing documents' contents.
 
-	/// Returns the message for the most recent error that has occured.
+	/// Returns the message for the most recent error that has occurred.
 	virtual std::string get_error(void) const;
 
     protected:
@@ -120,7 +120,7 @@ namespace Dijon
 	bool m_returnHeaders;
 	off_t m_maxSize;
 	const char *m_pData;
-	unsigned int m_dataLength;
+	off_t m_dataLength;
 	int m_fd;
 	GMimeStream *m_pGMimeMboxStream;
 	GMimeParser *m_pParser;
@@ -167,6 +167,8 @@ namespace Dijon
 
 	bool extractPart(GMimeObject *mimeObject, GMimeMboxPart &mboxPart);
 
+	bool extractDate(const std::string &header);
+
 	bool extractMessage(const std::string &subject);
 
 	void extractMetaData(GMimeMboxPart &mboxPart);
