@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2010 Fabrice Colin
+ *  Copyright 2007-2016 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,14 +32,14 @@ using std::endl;
 using namespace Dijon;
 
 #ifdef _DYNAMIC_DIJON_FILTERS
-DIJON_FILTER_EXPORT bool get_filter_types(std::set<std::string> &mime_types)
+DIJON_FILTER_EXPORT bool get_filter_types(MIMETypes &mime_types)
 {
-	mime_types.clear();
-	mime_types.insert("audio/mpeg");
-	mime_types.insert("audio/x-mp3");
-	mime_types.insert("application/ogg");
-	mime_types.insert("audio/x-flac+ogg");
-	mime_types.insert("audio/x-flac");
+	mime_types.m_mimeTypes.clear();
+	mime_types.m_mimeTypes.insert("audio/mpeg");
+	mime_types.m_mimeTypes.insert("audio/x-mp3");
+	mime_types.m_mimeTypes.insert("application/ogg");
+	mime_types.m_mimeTypes.insert("audio/x-flac+ogg");
+	mime_types.m_mimeTypes.insert("audio/x-flac");
 
 	return true;
 }
@@ -56,14 +56,14 @@ DIJON_FILTER_EXPORT bool check_filter_data_input(int data_input)
 	return false;
 }
 
-DIJON_FILTER_EXPORT Filter *get_filter(const std::string &mime_type)
+DIJON_FILTER_EXPORT Filter *get_filter(void)
 {
-	return new TagLibMusicFilter(mime_type);
+	return new TagLibMusicFilter();
 }
 #endif
 
-TagLibMusicFilter::TagLibMusicFilter(const string &mime_type) :
-	Filter(mime_type),
+TagLibMusicFilter::TagLibMusicFilter() :
+	Filter(),
 	m_parseDocument(false)
 {
 }
@@ -88,7 +88,7 @@ bool TagLibMusicFilter::set_property(Properties prop_name, const string &prop_va
 	return false;
 }
 
-bool TagLibMusicFilter::set_document_data(const char *data_ptr, unsigned int data_length)
+bool TagLibMusicFilter::set_document_data(const char *data_ptr, off_t data_length)
 {
 	return false;
 }
